﻿Module Module1

   Public UrlTemplate As String = ""
   Public UseSSO As Boolean = False

   Sub Main(ByVal Args() As String)
      Console.WriteLine("JComments to Disqus")
      Console.WriteLine("Version 1.0 written by Anders Hesselbom")
      Console.WriteLine()
      If Args Is Nothing Then
         ShowHelp()
         Return
      Else
         If Args.Length = 2 Then
            Dim Dom As New System.Xml.XmlDocument()
            Try
               Dom.Load(Args(0))
            Catch ex As System.Xml.XmlException
               Console.WriteLine("The document element of the source document must be renamed.")
               Console.WriteLine("Use Notepad or any XML editor to give the document element an")
               Console.WriteLine("XML compilant name. Make sure to also correct the closing tag.")
               Threading.Thread.Sleep(2000)
               Return
            Catch ex As Exception
               Console.WriteLine("Failed to load source file: """ & Args(0) & """.")
               Threading.Thread.Sleep(2000)
               Return
            End Try
            Dim Col As New ItemCollection()
            Console.WriteLine("Parsing...")
            Console.WriteLine()
            Col.Initialize(Dom)
            Console.WriteLine(Col.Count.ToString() & " records loaded.")
            If Col.Count <= 0 Then
               Console.WriteLine("Not enough data.")
               Threading.Thread.Sleep(2000)
               Return
            End If
            Console.WriteLine("Enter a URL template, use dollar sign")
            Console.WriteLine("for post ID. Example: http://www.blog.com/?p=$")
            Console.Write("URL Template>")
            UrlTemplate = Console.ReadLine().Trim()
            If UrlTemplate = "" Then
               Console.WriteLine("Interrupting. Bye.")
               Threading.Thread.Sleep(2000)
               Return
            End If
            Console.Write("Use SSO (Y/N)>")
            Dim SsoSvar As String = Console.ReadLine().Trim()
            If SsoSvar.ToLower().StartsWith("y") Then
               UseSSO = True
            End If
            Console.WriteLine()
            Dim Xml As String = Col.GetDisqusXml()
            Try
               Console.WriteLine("Creating destination file.")
               Using Sw As New System.IO.StreamWriter(Args(1), False, System.Text.Encoding.UTF8)
                  Sw.Write(Xml)
                  Sw.Flush()
                  Sw.Close()
               End Using
               Console.WriteLine("File converted successfully.")
               Threading.Thread.Sleep(2000)
               Return
            Catch ex As Exception
               Console.WriteLine("Failed to write XML. Make sure destination filename is correct and")
               Console.WriteLine("not locked by another program.")
               Threading.Thread.Sleep(2000)
               Return
            End Try
         Else
            ShowHelp()
         End If
      End If
   End Sub

   Private Sub ShowHelp()
      Console.WriteLine("Two arguments are required. One existing source file and a")
      Console.WriteLine("destination filename. Example:")
      Console.WriteLine()
      Console.WriteLine("js2disqus.exe ""C:\MyJCFile.xml"" ""C:\MyNewDisqusFile.xml""")
      Console.WriteLine()
      Console.WriteLine("The destination file will be overwritten.")
      Console.WriteLine()
      Threading.Thread.Sleep(2000)
   End Sub

End Module

Public Class Item

   Private mID As Integer
   Private mParentID As Integer
   Private mObjectID As Integer
   Private mObjectGroup As String
   Private mUserID As Integer
   Private mUserFullName As String
   Private mUsername As String
   Private mEMail As String
   Private mHomepage As String
   Private mCommentTitle As String
   Private mCommentText As String
   Private mIP As String
   Private mCommentDate As String
   Private mPublished As Boolean

   Public Sub New()
      Me.mID = 0
      Me.mParentID = 0
      Me.mObjectID = 0
      Me.mObjectGroup = ""
      Me.mUserID = 0
      Me.mUserFullName = ""
      Me.mUsername = ""
      Me.mEMail = ""
      Me.mHomepage = ""
      Me.mCommentTitle = ""
      Me.mCommentText = ""
      Me.mIP = ""
      Me.mCommentDate = ""
      Me.mPublished = False
   End Sub

   Public Sub New(ByVal X As System.Xml.XmlNode)
      Me.New()
      Me.ID = Me.ParseInt(Me.GetText(X, "id"))
      Me.ParentID = Me.ParseInt(Me.GetText(X, "parent"))
      Me.ObjectID = Me.ParseInt(Me.GetText(X, "object_id"))
      Me.ObjectGroup = Me.GetText(X, "object_group")
      Me.UserID = Me.ParseInt(Me.GetText(X, "userid"))
      Me.UserFullName = Me.GetText(X, "name")
      Me.Username = Me.GetText(X, "username")
      Me.EMail = Me.GetText(X, "email")
      Me.Homepage = Me.GetText(X, "homepage")
      Me.CommentTitle = Me.GetText(X, "title")
      Me.CommentText = Me.GetText(X, "comment")
      Me.IP = Me.GetText(X, "ip")
      Me.CommentDate = Me.GetText(X, "date")
      Me.Published = Me.ParseBool(Me.GetText(X, "published"))
   End Sub

   Private Function GetText(ByVal N As System.Xml.XmlNode, ByVal ChildName As String) As String
      If N.ChildNodes.Count > 0 Then
         For Each C As Xml.XmlNode In N.ChildNodes
            If String.Compare(C.Name, ChildName, True) = 0 Then
               Return C.InnerText
            End If
         Next
         Return ""
      Else
         Return ""
      End If
   End Function

   Private Function ParseInt(ByVal S As String) As Integer
      If S Is Nothing Then
         Return 0
      Else
         S = S.Trim()
         If S = "" Then
            Return 0
         Else
            Try
               Return Integer.Parse(S)
            Catch ex As Exception
               Return 0
            End Try
         End If
      End If
   End Function

   Private Function ParseBool(ByVal S As String) As Boolean
      If S Is Nothing Then
         Return False
      Else
         S = S.Trim()
         If S = "" Then
            Return False
         Else
            Select Case S.Substring(0, 1)
               Case "1", "y", "Y", "t", "T"
                  Return True
               Case Else
                  Return False
            End Select
         End If
      End If
   End Function

   Public Property ID As Integer
      Get
         Return Me.mID
      End Get
      Set(ByVal value As Integer)
         Me.mID = value
      End Set
   End Property

   Public Property ParentID As Integer
      Get
         Return Me.mParentID
      End Get
      Set(ByVal value As Integer)
         Me.mParentID = value
      End Set
   End Property

   Public Property ObjectID As Integer
      Get
         Return Me.mObjectID
      End Get
      Set(ByVal value As Integer)
         Me.mObjectID = value
      End Set
   End Property

   Public Property ObjectGroup As String
      Get
         Return Me.mObjectGroup
      End Get
      Set(ByVal value As String)
         Me.mObjectGroup = value
      End Set
   End Property

   Public Property UserID As Integer
      Get
         Return Me.mUserID
      End Get
      Set(ByVal value As Integer)
         Me.mUserID = value
      End Set
   End Property

   Public Property UserFullName As String
      Get
         Return Me.mUserFullName
      End Get
      Set(ByVal value As String)
         Me.mUserFullName = value
      End Set
   End Property

   Public Property Username As String
      Get
         Return Me.mUsername
      End Get
      Set(ByVal value As String)
         Me.mUsername = value
      End Set
   End Property

   Public Property EMail As String
      Get
         Return Me.mEMail
      End Get
      Set(ByVal value As String)
         Me.mEMail = value
      End Set
   End Property

   Public Property Homepage As String
      Get
         Return Me.mHomepage
      End Get
      Set(ByVal value As String)
         Me.mHomepage = value
      End Set
   End Property

   Public Property CommentTitle As String
      Get
         Return Me.mCommentTitle
      End Get
      Set(ByVal value As String)
         Me.mCommentTitle = value
      End Set
   End Property

   Public Property CommentText As String
      Get
         Return Me.mCommentText
      End Get
      Set(ByVal value As String)
         Me.mCommentText = value
      End Set
   End Property

   Public Property IP As String
      Get
         Return Me.mIP
      End Get
      Set(ByVal value As String)
         Me.mIP = value
      End Set
   End Property

   Public Property CommentDate As String
      Get
         Return Me.mCommentDate
      End Get
      Set(ByVal value As String)
         Me.mCommentDate = value
      End Set
   End Property

   Public Property Published() As Boolean
      Get
         Return Me.mPublished
      End Get
      Set(ByVal value As Boolean)
         Me.mPublished = value
      End Set
   End Property

   Public Sub GetDisqusXml(ByVal S As System.Text.StringBuilder)
      S.AppendLine("<item>")
      S.AppendLine("<title>" & System.Web.HttpUtility.HtmlEncode(Me.CommentTitle) & "</title>")
      S.AppendLine("<link>" & System.Web.HttpUtility.HtmlEncode(Module1.UrlTemplate.Replace("[ID]", Me.ObjectID.ToString()).Replace("[CAT]", Me.ParentID.ToString())) & "</link>")
      If CommentText = "" Then
         S.AppendLine("<content:encoded></content:encoded>")
      Else
         S.AppendLine("<content:encoded><![CDATA[" & System.Web.HttpUtility.HtmlEncode(Me.CommentText) & "]]></content:encoded>")
      End If
      S.AppendLine("<dsq:thread_identifier>" & System.Web.HttpUtility.HtmlEncode(Me.EMail) & "</dsq:thread_identifier>")
      S.AppendLine("<wp:post_date_gmt>" & Me.CommentDate & "</wp:post_date_gmt>")
      S.AppendLine("<wp:comment_status>open</wp:comment_status>")
      S.AppendLine("<wp:comment>")
      If Module1.UseSSO Then
         S.Append("<dsq:remote>")
         S.AppendLine("<dsq:id>" & Me.UserID.ToString() & "</dsq:id>")
         S.AppendLine("<dsq:avatar></dsq:avatar>")
         S.Append("</dsq:remote>")
      End If
      S.AppendLine("<wp:comment_id>" & Me.ID.ToString() & "</wp:comment_id>")
      S.AppendLine("<wp:comment_author>" & System.Web.HttpUtility.HtmlEncode(Me.UserFullName) & "</wp:comment_author>")
      S.AppendLine("<wp:comment_author_email>" & System.Web.HttpUtility.HtmlEncode(Me.EMail) & "</wp:comment_author_email>")
      S.AppendLine("<wp:comment_author_url>" & System.Web.HttpUtility.HtmlEncode(Me.Homepage) & "</wp:comment_author_url>")
      S.AppendLine("<wp:comment_author_IP>" & Me.IP & "</wp:comment_author_IP>")
      S.AppendLine("<wp:comment_date_gmt>" & Me.CommentDate & "</wp:comment_date_gmt>")
      If CommentText = "" Then
         S.AppendLine("<wp:comment_content></wp:comment_content>")
      Else
         S.AppendLine("<wp:comment_content><![CDATA[" & System.Web.HttpUtility.HtmlEncode(Me.CommentText) & "]]></wp:comment_content>")
      End If
      If Me.Published Then
         S.AppendLine("<wp:comment_approved>1</wp:comment_approved>")
      Else
         S.AppendLine("<wp:comment_approved>0</wp:comment_approved>")
      End If
      S.AppendLine("<wp:comment_parent>" & Me.ParentID.ToString() & "</wp:comment_parent>")
      S.AppendLine("</wp:comment>")
      S.AppendLine("</item>")
   End Sub

End Class

Public Class ItemCollection
   Inherits CollectionBase

   Public Function Add(ByVal I As Item) As Integer
      Return List.Add(I)
   End Function

   Default Public ReadOnly Property Item(ByVal Index As Integer) As Item
      Get
         Return CType(List(Index), Item)
      End Get
   End Property

   Public Sub Initialize(ByVal Dom As System.Xml.XmlDocument)
      If Dom.DocumentElement.ChildNodes.Count > 0 Then
         For I As Integer = 0 To Dom.DocumentElement.ChildNodes.Count - 1
            Console.CursorTop -= 1
            Console.WriteLine(Me.GetPercent(I, Dom.DocumentElement.ChildNodes.Count).ToString() & "%")
            Me.Add(New Item(Dom.DocumentElement.ChildNodes(I)))
         Next
      End If
   End Sub

   Private Function GetPercent(ByVal N As Integer, ByVal Mx As Integer) As Integer
      If N <= 0 Then
         Return 0
      ElseIf N >= (Mx - 1) Then
         Return 100
      Else
         Try
            Return CType(Math.Round((N / (Mx - 1)) * 100, 0), Integer)
         Catch ex As Exception
            Return 0
         End Try
      End If
   End Function

   Public Function GetDisqusXml() As String
      Dim S As New System.Text.StringBuilder()
      S.AppendLine("<?xml version=""1.0"" encoding=""UTF-8""?>")
      S.AppendLine("<rss version=""2.0"" xmlns:content=""http://purl.org/rss/1.0/modules/content/"" xmlns:dsq=""http://www.disqus.com/"" xmlns:dc = ""http://purl.org/dc/elements/1.1/"" xmlns:wp=""http://wordpress.org/export/1.0/"">")
      S.AppendLine("<channel>")
      Dim Count As Integer = 0
      For Each I As Item In Me
         Count += 1
         Console.CursorTop -= 1
         Console.WriteLine(Count)
         I.GetDisqusXml(S)
      Next
      S.AppendLine("</channel>")
      S.AppendLine("</rss>")
      Return S.ToString()
   End Function

End Class
